#include "HelloWorldScene.h"

#include "generic-box2d-plist/GB2ShapeCache-x.h"
#include "Box2D/Box2D.h"

USING_NS_CC;

#define PTM_RATIO 32

Scene* HelloWorld::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    if (!Layer::init()) {
        return false;
    }
    
    Size visibleSize = Director::getInstance()->getVisibleSize();
    
    // 作成したPhysicsEditorのファイルを読込み
    GB2ShapeCache::sharedGB2ShapeCache()->addShapesWithFile("shapedefs.plist");
    
    // 重力の設定
    b2Vec2 gravity;
    gravity.Set(0.0f, -10.0f);
    
    // Worldを作成
    _world = new b2World(gravity);
    _world->SetAllowSleeping(true);
    _world->SetContinuousPhysics(true);
    
    // Bodyの作成
    b2BodyDef groundBodyDef;
    groundBodyDef.position.Set(visibleSize.width/2/PTM_RATIO,
                               visibleSize.height/2/PTM_RATIO);
    auto groundBody = _world->CreateBody(&groundBodyDef);
    
    // 上下左右に壁を配置
    b2PolygonShape groundBox;
    // 下
    groundBox.SetAsBox(visibleSize.width/2/PTM_RATIO, 0, b2Vec2(0, -visibleSize.height/2/PTM_RATIO), 0);
    groundBody->CreateFixture(&groundBox, 0);
    // 上
    groundBox.SetAsBox(visibleSize.width/2/PTM_RATIO, 0, b2Vec2(0, visibleSize.height/2/PTM_RATIO), 0);
    groundBody->CreateFixture(&groundBox, 0);
    // 左
    groundBox.SetAsBox(0, visibleSize.height/2/PTM_RATIO, b2Vec2(-visibleSize.width/2/PTM_RATIO, 0), 0);
    groundBody->CreateFixture(&groundBox, 0);
    // 右
    groundBox.SetAsBox(0, visibleSize.height/2/PTM_RATIO, b2Vec2(visibleSize.width/2/PTM_RATIO, 0), 0);
    groundBody->CreateFixture(&groundBox, 0);
    
    // 当たり判定を設定したオブジェクトの追加
    addNewSpriteWithCoords(Point(visibleSize.width/2, visibleSize.height/2));
    
    // Box2Dのオブジェクトの更新
    schedule(schedule_selector(HelloWorld::tick));
    
    // 画面をタップした時に、オブジェクトを追加する
    auto listener = EventListenerTouchOneByOne::create();
    listener->onTouchBegan = [&](Touch *touch, Event *event) -> bool {
        this->addNewSpriteWithCoords(touch->getLocation());
        return true;
    };
    this->getEventDispatcher()->addEventListenerWithSceneGraphPriority(listener, this);
    
    return true;
}

void HelloWorld::addNewSpriteWithCoords(Point p)
{
    std::string name = "grossini";
    
    // スプライトを読込み
    auto sprite = Sprite::create((name+".png").c_str());
    sprite->setPosition(p);
    this->addChild(sprite);
    
    // Body設定
    b2BodyDef bodyDef;
    bodyDef.type = b2_dynamicBody;
    bodyDef.position.Set(p.x/PTM_RATIO, p.y/PTM_RATIO);
    bodyDef.userData = sprite;
    auto body = _world->CreateBody(&bodyDef);
    
    // PhysicsEditorで作成したファイルを元にオブジェクトを生成
    auto sc = GB2ShapeCache::sharedGB2ShapeCache();
    sc->addFixturesToBody(body, name.c_str());
    sprite->setAnchorPoint(sc->anchorPointForShape(name.c_str()));
}

void HelloWorld::tick(float dt)
{
    // Worldの更新
    int velocityIterations = 8;
    int positionIterations = 1;
    _world->Step(dt, velocityIterations, positionIterations);
    
    // Bodyの更新
    for (auto b = _world->GetBodyList(); b; b = b->GetNext()) {
        if (b->GetUserData() != NULL) {
            auto actor = (Sprite *)b->GetUserData();
            actor->setPosition(Vec2(b->GetPosition().x * PTM_RATIO, b->GetPosition().y * PTM_RATIO));
            actor->setRotation(-1 * CC_RADIANS_TO_DEGREES(b->GetAngle()));
        }
    }
}
