/**
 *
 *  You can modify and use this source freely
 *  only for the development of application related Live2D.
 *
 *  (c) Live2D Inc. All rights reserved.
 */
#include "SampleLive2DSprite.h"
#include "util/UtSystem.h"

#include "graphics/DrawProfileCocos2D.h"
#include "platform/CCFileUtils.h"

using namespace live2d;
USING_NS_CC;

SampleLive2DSprite::SampleLive2DSprite()
{
  //Live2D Sample
  const char* MODEL = "Epsilon_free.moc" ;
  const char* TEXTURES[] ={
    "texture_00.png",
    NULL
  } ;
  unsigned char* buf;
  ssize_t bufSize;
  buf = FileUtils::getInstance()->getFileData(MODEL,"rb", &bufSize);
  
  live2DModel = Live2DModelOpenGL::loadModel( buf,bufSize ) ;
  
  for( int i = 0 ; TEXTURES[i] != NULL ; i++ ){
    Texture2D *texture = new Texture2D();
    Image *img = new Image();
    
    img->initWithImageFile(TEXTURES[i]);
    texture->initWithImage(img);
    
    
    Texture2D::TexParams texParams = { GL_LINEAR_MIPMAP_LINEAR, GL_LINEAR_MIPMAP_LINEAR, GL_CLAMP_TO_EDGE, GL_CLAMP_TO_EDGE };
    texture->setTexParameters(texParams);
    texture->generateMipmap();
    
    int glTexNo = texture->getName() ;
    
    live2DModel->setTexture( i , glTexNo ) ;
    textures.push_back( texture ) ;
  }
  
  float w = Director::getInstance()->getWinSize().width;
  float h = Director::getInstance()->getWinSize().height;
  float scx = 2.0 / live2DModel->getCanvasWidth() ;
  float scy = -2.0 / live2DModel->getCanvasWidth() * (w/h);
  float x = -1 ;
  float y = 1 ;
  float matrix []= {
    scx , 0 , 0 , 0 ,
    0 , scy ,0 , 0 ,
    0 , 0 , 1 , 0 ,
    x , y , 0 , 1
  } ;
  
  live2DModel->setMatrix(matrix) ;
  live2DModel->setPremultipliedAlpha( true );
}

SampleLive2DSprite::~SampleLive2DSprite()
{
  delete live2DModel;

  for (int i=0; i<textures.size(); i++)
  {
    textures[i]->release();
  }
}


void SampleLive2DSprite::draw(cocos2d::Renderer *renderer, const cocos2d::Mat4 &transform, uint32_t transformUpdated)
{
  Sprite::draw(renderer, transform, transformUpdated);
  
  
  _customCommand.init(_globalZOrder);
  _customCommand.func = CC_CALLBACK_0(SampleLive2DSprite::onDraw, this, transform, transformUpdated);
  renderer->addCommand(&_customCommand);
}

void SampleLive2DSprite::onDraw(const kmMat4 &transform, bool transformUpdated)
{
  kmGLPushMatrix();
  kmGLLoadMatrix(&transform);
  
  double t = (UtSystem::getUserTimeMSec()/1000.0) * 2 * M_PI  ;
  double cycle=3.0;
  double value=sin( t/cycle );
  live2DModel->setParamFloat( "PARAM_ANGLE_X" , (float) (30 * value) ) ;
  
  live2d::DrawProfileCocos2D::preDraw();
  
  live2DModel->update() ;
  live2DModel->draw() ;
  
  live2d::DrawProfileCocos2D::postDraw() ;
  
  kmGLPopMatrix();
}
